import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";

var _SHAPE_TO_TAGS, _dec, _dec2, _dec3, _dec4, _class, _class2, _descriptor, _descriptor2, _descriptor3, _class3, _temp;

import { inject, injectable } from 'inversify';
import { vec3 } from 'gl-matrix';
import { ContextService, SceneGraphService, SHAPE, fromRotationTranslationScale, getEuler, Renderable } from '@antv/g';
import { ElementSVG } from './components/ElementSVG';
import { createSVGElement } from './utils/dom';
import { ElementRendererFactory } from './shapes/paths';
export var SHAPE_TO_TAGS = (_SHAPE_TO_TAGS = {}, _SHAPE_TO_TAGS[SHAPE.Rect] = 'path', _SHAPE_TO_TAGS[SHAPE.Circle] = 'circle', _SHAPE_TO_TAGS[SHAPE.Ellipse] = 'ellipse', _SHAPE_TO_TAGS[SHAPE.Image] = 'image', _SHAPE_TO_TAGS[SHAPE.Group] = 'g', _SHAPE_TO_TAGS[SHAPE.Line] = 'line', _SHAPE_TO_TAGS[SHAPE.Polyline] = 'polyline', _SHAPE_TO_TAGS[SHAPE.Polygon] = 'polygon', _SHAPE_TO_TAGS[SHAPE.Text] = 'text', _SHAPE_TO_TAGS);
export var SVG_ATTR_MAP = {
  opacity: 'opacity',
  fillStyle: 'fill',
  fill: 'fill',
  fillOpacity: 'fill-opacity',
  strokeStyle: 'stroke',
  strokeOpacity: 'stroke-opacity',
  stroke: 'stroke',
  r: 'r',
  rx: 'rx',
  ry: 'ry',
  width: 'width',
  height: 'height',
  // x1: 'x1',
  // x2: 'x2',
  // y1: 'y1',
  // y2: 'y2',
  lineCap: 'stroke-linecap',
  lineJoin: 'stroke-linejoin',
  lineWidth: 'stroke-width',
  lineDash: 'stroke-dasharray',
  lineDashOffset: 'stroke-dashoffset',
  miterLimit: 'stroke-miterlimit',
  font: 'font',
  fontSize: 'font-size',
  fontStyle: 'font-style',
  fontVariant: 'font-variant',
  fontWeight: 'font-weight',
  fontFamily: 'font-family',
  letterSpacing: 'letter-spacing',
  startArrow: 'marker-start',
  endArrow: 'marker-end',
  path: 'd',
  class: 'class',
  id: 'id',
  style: 'style',
  preserveAspectRatio: 'preserveAspectRatio',
  visibility: 'visibility'
};
export var SVGRendererPlugin = (_dec = injectable(), _dec2 = inject(ContextService), _dec3 = inject(SceneGraphService), _dec4 = inject(ElementRendererFactory), _dec(_class = (_class2 = (_temp = _class3 = /*#__PURE__*/function () {
  function SVGRendererPlugin() {
    _initializerDefineProperty(this, "contextService", _descriptor, this);

    _initializerDefineProperty(this, "sceneGraphService", _descriptor2, this);

    _initializerDefineProperty(this, "elementRendererFactory", _descriptor3, this);
  }

  var _proto = SVGRendererPlugin.prototype;

  _proto.apply = function apply(renderingService) {
    var _this = this;

    renderingService.hooks.mounted.tap(SVGRendererPlugin.tag, function (object) {
      var entity = object.getEntity(); // create svg element

      var svgElement = entity.addComponent(ElementSVG);
      var type = SHAPE_TO_TAGS[object.nodeType];

      if (type) {
        var _object$parentNode;

        var $groupEl;
        var $el = createSVGElement(type);
        $el.id = entity.getName();

        if (type !== 'g') {
          $groupEl = createSVGElement('g');
          $groupEl.appendChild($el);
        } else {
          $groupEl = $el;
        }

        svgElement.$el = $el;
        svgElement.$groupEl = $groupEl;

        var $parentGroupEl = ((_object$parentNode = object.parentNode) === null || _object$parentNode === void 0 ? void 0 : _object$parentNode.getEntity().getComponent(ElementSVG).$groupEl) || _this.contextService.getContext();

        if ($parentGroupEl) {
          $parentGroupEl.appendChild($groupEl);
        }
      }
    });
    renderingService.hooks.unmounted.tap(SVGRendererPlugin.tag, function (object) {
      var element = object.getEntity().getComponent(ElementSVG);

      if (element) {
        element.$el = null;
      }
    });
    renderingService.hooks.render.tap(SVGRendererPlugin.tag, function (objects) {
      objects.forEach(function (object) {
        var entity = object.getEntity();
        var $el = entity.getComponent(ElementSVG).$el;
        var $groupEl = entity.getComponent(ElementSVG).$groupEl;

        if ($el && $groupEl) {
          var nodeType = object.nodeType,
              attributes = object.attributes; // apply local RTS transformation to <group> wrapper

          _this.applyTransform($groupEl, object);

          $el.setAttribute('fill', 'none');

          if (nodeType === SHAPE.Image) {
            $el.setAttribute('preserveAspectRatio', 'none');
          } // apply attributes


          for (var name in attributes) {
            _this.updateAttribute(entity, name, "" + attributes[name]);
          } // generate path


          var elementRenderer = _this.elementRendererFactory(nodeType);

          if (elementRenderer) {
            elementRenderer.apply($el, attributes);
          }
        } // finish rendering, clear dirty flag


        var renderable = entity.getComponent(Renderable);
        renderable.dirty = false;
      });
    });
    renderingService.hooks.attributeChanged.tap(SVGRendererPlugin.tag, function (object, name, value) {
      var entity = object.getEntity();

      if (name === 'z-index') {
        var _parentEntity$getComp;

        var parent = object.parentNode;
        var parentEntity = object.parentNode.getEntity();
        var $groupEl = parentEntity === null || parentEntity === void 0 ? void 0 : (_parentEntity$getComp = parentEntity.getComponent(ElementSVG)) === null || _parentEntity$getComp === void 0 ? void 0 : _parentEntity$getComp.$groupEl;
        var children = [].concat(parent.children);

        if ($groupEl) {
          // need to reorder parent's children
          children.sort(_this.sceneGraphService.sort); // create empty fragment

          var fragment = document.createDocumentFragment();
          children.forEach(function (child) {
            var $el = child.getEntity().getComponent(ElementSVG).$groupEl;

            if ($el) {
              fragment.appendChild($el);
            }
          });
          $groupEl.appendChild(fragment);
        }
      }

      _this.updateAttribute(entity, name, value);
    });
  };

  _proto.applyTransform = function applyTransform($el, object) {
    var _getEuler = getEuler(vec3.create(), object.getLocalRotation()),
        ex = _getEuler[0],
        ey = _getEuler[1],
        ez = _getEuler[2];

    var _object$getLocalPosit = object.getLocalPosition(),
        x = _object$getLocalPosit[0],
        y = _object$getLocalPosit[1];

    var _object$getLocalScale = object.getLocalScale(),
        scaleX = _object$getLocalScale[0],
        scaleY = _object$getLocalScale[1]; // gimbal lock at 90 degrees


    var rts = fromRotationTranslationScale(ex || ez, x, y, scaleX, scaleY); // TODO: use proper precision avoiding too long string in `transform`
    // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations

    $el.setAttribute('transform', "matrix(" + rts[0] + "," + rts[1] + "," + rts[3] + "," + rts[4] + "," + rts[6] + "," + rts[7] + ")");
  };

  _proto.updateAttribute = function updateAttribute(entity, name, value) {
    if (SVG_ATTR_MAP[name]) {
      // update `visibility` on <group>
      if (name === 'visibility') {
        var _entity$getComponent, _entity$getComponent$;

        (_entity$getComponent = entity.getComponent(ElementSVG)) === null || _entity$getComponent === void 0 ? void 0 : (_entity$getComponent$ = _entity$getComponent.$groupEl) === null || _entity$getComponent$ === void 0 ? void 0 : _entity$getComponent$.setAttribute(SVG_ATTR_MAP[name], "" + value);
      } else {
        var _entity$getComponent2, _entity$getComponent3;

        (_entity$getComponent2 = entity.getComponent(ElementSVG)) === null || _entity$getComponent2 === void 0 ? void 0 : (_entity$getComponent3 = _entity$getComponent2.$el) === null || _entity$getComponent3 === void 0 ? void 0 : _entity$getComponent3.setAttribute(SVG_ATTR_MAP[name], "" + value);
      }
    }
  };

  return SVGRendererPlugin;
}(), _class3.tag = 'SVGRendererPlugin', _temp), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "contextService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class2.prototype, "sceneGraphService", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class2.prototype, "elementRendererFactory", [_dec4], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
//# sourceMappingURL=SVGRendererPlugin.js.map