import { ContainerModule } from 'inversify';
import { RenderingPluginContribution, SHAPE, world } from '@antv/g';
import { ElementSVG } from './components/ElementSVG';
import { ElementRendererFactory, ImageRenderer, LineRenderer, PolylineRenderer, RectRenderer, TextRenderer } from './shapes/paths';
import { SVGRendererPlugin } from './SVGRendererPlugin';
world.registerComponent(ElementSVG);
export { ElementSVG };
export var containerModule = new ContainerModule(function (bind, unbind, isBound, rebind) {
  /**
   * register shape renderers
   */
  bind(RectRenderer).toSelf().inSingletonScope();
  bind(ImageRenderer).toSelf().inSingletonScope();
  bind(LineRenderer).toSelf().inSingletonScope();
  bind(PolylineRenderer).toSelf().inSingletonScope();
  bind(TextRenderer).toSelf().inSingletonScope();
  bind(ElementRendererFactory).toFactory(function (ctx) {
    return function (tagName) {
      if (tagName === SHAPE.Rect) {
        return ctx.container.get(RectRenderer);
      } else if (tagName === SHAPE.Image) {
        return ctx.container.get(ImageRenderer);
      } else if (tagName === SHAPE.Line) {
        return ctx.container.get(LineRenderer);
      } else if (tagName === SHAPE.Polyline || tagName === SHAPE.Polygon) {
        return ctx.container.get(PolylineRenderer);
      } else if (tagName === SHAPE.Text) {
        return ctx.container.get(TextRenderer);
      }

      return null;
    };
  });
  bind(SVGRendererPlugin).toSelf().inSingletonScope();
  bind(RenderingPluginContribution).toService(SVGRendererPlugin);
});
//# sourceMappingURL=index.js.map