import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";

var _dec, _dec2, _class, _class2, _descriptor;

import { TextService } from '@antv/g';
import { detect } from 'detect-browser';
import { inject, injectable } from 'inversify';
// @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Attribute/alignment-baseline
var BASELINE_MAP = {
  top: 'before-edge',
  // TODO: different with canvas' `top`
  middle: 'central',
  bottom: 'after-edge',
  alphabetic: 'alphabetic',
  ideographic: 'ideographic',
  hanging: 'hanging'
}; // for FireFox
// @see https://github.com/plouc/nivo/issues/164

var BASELINE_MAP_FOR_FIREFOX = {
  top: 'text-before-edge',
  middle: 'central',
  bottom: 'text-after-edge',
  alphabetic: 'alphabetic',
  ideographic: 'ideographic',
  hanging: 'hanging'
};
var ANCHOR_MAP = {
  left: 'left',
  start: 'left',
  center: 'middle',
  right: 'end',
  end: 'end'
};
export var TextRenderer = (_dec = injectable(), _dec2 = inject(TextService), _dec(_class = (_class2 = /*#__PURE__*/function () {
  function TextRenderer() {
    _initializerDefineProperty(this, "textService", _descriptor, this);
  }

  var _proto = TextRenderer.prototype;

  _proto.apply = function apply($el, attributes) {
    var textAlign = attributes.textAlign,
        text = attributes.text,
        textBaseline = attributes.textBaseline,
        _attributes$fontSize = attributes.fontSize,
        fontSize = _attributes$fontSize === void 0 ? 0 : _attributes$fontSize,
        lineCap = attributes.lineCap,
        lineJoin = attributes.lineJoin,
        _attributes$lineWidth = attributes.lineWidth,
        lineWidth = _attributes$lineWidth === void 0 ? 0 : _attributes$lineWidth;
    var browser = detect();

    if (browser && browser.name === 'firefox') {
      // compatible with FireFox browser, ref: https://github.com/antvis/g/issues/119
      $el.setAttribute('dominant-baseline', BASELINE_MAP_FOR_FIREFOX[textBaseline] || 'alphabetic');
    } else {
      $el.setAttribute('dominant-baseline', BASELINE_MAP_FOR_FIREFOX[textBaseline]);
      $el.setAttribute('alignment-baseline', BASELINE_MAP[textBaseline]);
    }

    $el.setAttribute('text-anchor', ANCHOR_MAP[textAlign]);
    $el.setAttribute('paint-order', 'stroke');
    $el.setAttribute('style', "stroke-linecap:" + lineCap + "; stroke-linejoin:" + lineJoin + ";");

    var _this$textService$mea = this.textService.measureText(text, attributes),
        lines = _this$textService$mea.lines,
        lineHeight = _this$textService$mea.lineHeight,
        height = _this$textService$mea.height;

    var lineNum = lines.length;

    if (lineNum === 1) {
      $el.innerHTML = lines[0];
      $el.setAttribute('dx', "" + lineWidth / 2);
    } else {
      $el.innerHTML = lines.map(function (line, i) {
        var dx = lineWidth / 2;
        var dy = 0;

        if (i === 0) {
          // TODO: handle other textBaseline values
          if (textBaseline === 'middle') {
            dy = lineHeight / 2 - height / 2;
          } else if (textBaseline === 'top' || textBaseline === 'hanging') {
            dy = 0;
          } else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
            dy = -lineHeight * (lineNum - 1);
          }
        } else {
          dy = lineHeight;
        }

        return "<tspan x=\"0\" dx=\"" + dx + "\" dy=\"" + dy + "\">" + line + "</tspan>";
      }).join('');
    }
  };

  return TextRenderer;
}(), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "textService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
//# sourceMappingURL=Text.js.map