"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var _a;
exports.__esModule = true;
exports.SVGRendererPlugin = exports.SVG_ATTR_MAP = exports.SHAPE_TO_TAGS = void 0;
var inversify_1 = require("inversify");
var gl_matrix_1 = require("gl-matrix");
var g_1 = require("@antv/g");
var ElementSVG_1 = require("./components/ElementSVG");
var dom_1 = require("./utils/dom");
var paths_1 = require("./shapes/paths");
exports.SHAPE_TO_TAGS = (_a = {},
    _a[g_1.SHAPE.Rect] = 'path',
    _a[g_1.SHAPE.Circle] = 'circle',
    _a[g_1.SHAPE.Ellipse] = 'ellipse',
    _a[g_1.SHAPE.Image] = 'image',
    _a[g_1.SHAPE.Group] = 'g',
    _a[g_1.SHAPE.Line] = 'line',
    _a[g_1.SHAPE.Polyline] = 'polyline',
    _a[g_1.SHAPE.Polygon] = 'polygon',
    _a[g_1.SHAPE.Text] = 'text',
    _a);
exports.SVG_ATTR_MAP = {
    opacity: 'opacity',
    fillStyle: 'fill',
    fill: 'fill',
    fillOpacity: 'fill-opacity',
    strokeStyle: 'stroke',
    strokeOpacity: 'stroke-opacity',
    stroke: 'stroke',
    r: 'r',
    rx: 'rx',
    ry: 'ry',
    width: 'width',
    height: 'height',
    // x1: 'x1',
    // x2: 'x2',
    // y1: 'y1',
    // y2: 'y2',
    lineCap: 'stroke-linecap',
    lineJoin: 'stroke-linejoin',
    lineWidth: 'stroke-width',
    lineDash: 'stroke-dasharray',
    lineDashOffset: 'stroke-dashoffset',
    miterLimit: 'stroke-miterlimit',
    font: 'font',
    fontSize: 'font-size',
    fontStyle: 'font-style',
    fontVariant: 'font-variant',
    fontWeight: 'font-weight',
    fontFamily: 'font-family',
    letterSpacing: 'letter-spacing',
    startArrow: 'marker-start',
    endArrow: 'marker-end',
    path: 'd',
    "class": 'class',
    id: 'id',
    style: 'style',
    preserveAspectRatio: 'preserveAspectRatio',
    visibility: 'visibility'
};
var SVGRendererPlugin = /** @class */ (function () {
    function SVGRendererPlugin() {
    }
    SVGRendererPlugin_1 = SVGRendererPlugin;
    SVGRendererPlugin.prototype.apply = function (renderingService) {
        var _this = this;
        renderingService.hooks.mounted.tap(SVGRendererPlugin_1.tag, function (object) {
            var _a;
            var entity = object.getEntity();
            // create svg element
            var svgElement = entity.addComponent(ElementSVG_1.ElementSVG);
            var type = exports.SHAPE_TO_TAGS[object.nodeType];
            if (type) {
                var $groupEl = void 0;
                var $el = dom_1.createSVGElement(type);
                $el.id = entity.getName();
                if (type !== 'g') {
                    $groupEl = dom_1.createSVGElement('g');
                    $groupEl.appendChild($el);
                }
                else {
                    $groupEl = $el;
                }
                svgElement.$el = $el;
                svgElement.$groupEl = $groupEl;
                var $parentGroupEl = ((_a = object.parentNode) === null || _a === void 0 ? void 0 : _a.getEntity().getComponent(ElementSVG_1.ElementSVG).$groupEl) || _this.contextService.getContext();
                if ($parentGroupEl) {
                    $parentGroupEl.appendChild($groupEl);
                }
            }
        });
        renderingService.hooks.unmounted.tap(SVGRendererPlugin_1.tag, function (object) {
            var element = object.getEntity().getComponent(ElementSVG_1.ElementSVG);
            if (element) {
                element.$el = null;
            }
        });
        renderingService.hooks.render.tap(SVGRendererPlugin_1.tag, function (objects) {
            objects.forEach(function (object) {
                var entity = object.getEntity();
                var $el = entity.getComponent(ElementSVG_1.ElementSVG).$el;
                var $groupEl = entity.getComponent(ElementSVG_1.ElementSVG).$groupEl;
                if ($el && $groupEl) {
                    var nodeType = object.nodeType, attributes = object.attributes;
                    // apply local RTS transformation to <group> wrapper
                    _this.applyTransform($groupEl, object);
                    $el.setAttribute('fill', 'none');
                    if (nodeType === g_1.SHAPE.Image) {
                        $el.setAttribute('preserveAspectRatio', 'none');
                    }
                    // apply attributes
                    for (var name_1 in attributes) {
                        _this.updateAttribute(entity, name_1, "" + attributes[name_1]);
                    }
                    // generate path
                    var elementRenderer = _this.elementRendererFactory(nodeType);
                    if (elementRenderer) {
                        elementRenderer.apply($el, attributes);
                    }
                }
                // finish rendering, clear dirty flag
                var renderable = entity.getComponent(g_1.Renderable);
                renderable.dirty = false;
            });
        });
        renderingService.hooks.attributeChanged.tap(SVGRendererPlugin_1.tag, function (object, name, value) {
            var _a;
            var entity = object.getEntity();
            if (name === 'z-index') {
                var parent_1 = object.parentNode;
                var parentEntity = object.parentNode.getEntity();
                var $groupEl = (_a = parentEntity === null || parentEntity === void 0 ? void 0 : parentEntity.getComponent(ElementSVG_1.ElementSVG)) === null || _a === void 0 ? void 0 : _a.$groupEl;
                var children = __spreadArrays(parent_1.children);
                if ($groupEl) {
                    // need to reorder parent's children
                    children.sort(_this.sceneGraphService.sort);
                    // create empty fragment
                    var fragment_1 = document.createDocumentFragment();
                    children.forEach(function (child) {
                        var $el = child.getEntity().getComponent(ElementSVG_1.ElementSVG).$groupEl;
                        if ($el) {
                            fragment_1.appendChild($el);
                        }
                    });
                    $groupEl.appendChild(fragment_1);
                }
            }
            _this.updateAttribute(entity, name, value);
        });
    };
    SVGRendererPlugin.prototype.applyTransform = function ($el, object) {
        var _a = g_1.getEuler(gl_matrix_1.vec3.create(), object.getLocalRotation()), ex = _a[0], ey = _a[1], ez = _a[2];
        var _b = object.getLocalPosition(), x = _b[0], y = _b[1];
        var _c = object.getLocalScale(), scaleX = _c[0], scaleY = _c[1];
        // gimbal lock at 90 degrees
        var rts = g_1.fromRotationTranslationScale(ex || ez, x, y, scaleX, scaleY);
        // TODO: use proper precision avoiding too long string in `transform`
        // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations
        $el.setAttribute('transform', "matrix(" + rts[0] + "," + rts[1] + "," + rts[3] + "," + rts[4] + "," + rts[6] + "," + rts[7] + ")");
    };
    SVGRendererPlugin.prototype.updateAttribute = function (entity, name, value) {
        var _a, _b, _c, _d;
        if (exports.SVG_ATTR_MAP[name]) {
            // update `visibility` on <group>
            if (name === 'visibility') {
                (_b = (_a = entity.getComponent(ElementSVG_1.ElementSVG)) === null || _a === void 0 ? void 0 : _a.$groupEl) === null || _b === void 0 ? void 0 : _b.setAttribute(exports.SVG_ATTR_MAP[name], "" + value);
            }
            else {
                (_d = (_c = entity.getComponent(ElementSVG_1.ElementSVG)) === null || _c === void 0 ? void 0 : _c.$el) === null || _d === void 0 ? void 0 : _d.setAttribute(exports.SVG_ATTR_MAP[name], "" + value);
            }
        }
    };
    var SVGRendererPlugin_1;
    SVGRendererPlugin.tag = 'SVGRendererPlugin';
    __decorate([
        inversify_1.inject(g_1.ContextService)
    ], SVGRendererPlugin.prototype, "contextService");
    __decorate([
        inversify_1.inject(g_1.SceneGraphService)
    ], SVGRendererPlugin.prototype, "sceneGraphService");
    __decorate([
        inversify_1.inject(paths_1.ElementRendererFactory)
    ], SVGRendererPlugin.prototype, "elementRendererFactory");
    SVGRendererPlugin = SVGRendererPlugin_1 = __decorate([
        inversify_1.injectable()
    ], SVGRendererPlugin);
    return SVGRendererPlugin;
}());
exports.SVGRendererPlugin = SVGRendererPlugin;
