"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.__esModule = true;
exports.TextRenderer = void 0;
var g_1 = require("@antv/g");
var detect_browser_1 = require("detect-browser");
var inversify_1 = require("inversify");
// @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Attribute/alignment-baseline
var BASELINE_MAP = {
    top: 'before-edge',
    middle: 'central',
    bottom: 'after-edge',
    alphabetic: 'alphabetic',
    ideographic: 'ideographic',
    hanging: 'hanging'
};
// for FireFox
// @see https://github.com/plouc/nivo/issues/164
var BASELINE_MAP_FOR_FIREFOX = {
    top: 'text-before-edge',
    middle: 'central',
    bottom: 'text-after-edge',
    alphabetic: 'alphabetic',
    ideographic: 'ideographic',
    hanging: 'hanging'
};
var ANCHOR_MAP = {
    left: 'left',
    start: 'left',
    center: 'middle',
    right: 'end',
    end: 'end'
};
var TextRenderer = /** @class */ (function () {
    function TextRenderer() {
    }
    TextRenderer.prototype.apply = function ($el, attributes) {
        var textAlign = attributes.textAlign, text = attributes.text, textBaseline = attributes.textBaseline, _a = attributes.fontSize, fontSize = _a === void 0 ? 0 : _a, lineCap = attributes.lineCap, lineJoin = attributes.lineJoin, _b = attributes.lineWidth, lineWidth = _b === void 0 ? 0 : _b;
        var browser = detect_browser_1.detect();
        if (browser && browser.name === 'firefox') {
            // compatible with FireFox browser, ref: https://github.com/antvis/g/issues/119
            $el.setAttribute('dominant-baseline', BASELINE_MAP_FOR_FIREFOX[textBaseline] || 'alphabetic');
        }
        else {
            $el.setAttribute('dominant-baseline', BASELINE_MAP_FOR_FIREFOX[textBaseline]);
            $el.setAttribute('alignment-baseline', BASELINE_MAP[textBaseline]);
        }
        $el.setAttribute('text-anchor', ANCHOR_MAP[textAlign]);
        $el.setAttribute('paint-order', 'stroke');
        $el.setAttribute('style', "stroke-linecap:" + lineCap + "; stroke-linejoin:" + lineJoin + ";");
        var _c = this.textService.measureText(text, attributes), lines = _c.lines, lineHeight = _c.lineHeight, height = _c.height;
        var lineNum = lines.length;
        if (lineNum === 1) {
            $el.innerHTML = lines[0];
            $el.setAttribute('dx', "" + lineWidth / 2);
        }
        else {
            $el.innerHTML = lines
                .map(function (line, i) {
                var dx = lineWidth / 2;
                var dy = 0;
                if (i === 0) {
                    // TODO: handle other textBaseline values
                    if (textBaseline === 'middle') {
                        dy = lineHeight / 2 - height / 2;
                    }
                    else if (textBaseline === 'top' || textBaseline === 'hanging') {
                        dy = 0;
                    }
                    else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
                        dy = -lineHeight * (lineNum - 1);
                    }
                }
                else {
                    dy = lineHeight;
                }
                return "<tspan x=\"0\" dx=\"" + dx + "\" dy=\"" + dy + "\">" + line + "</tspan>";
            })
                .join('');
        }
    };
    __decorate([
        inversify_1.inject(g_1.TextService)
    ], TextRenderer.prototype, "textService");
    TextRenderer = __decorate([
        inversify_1.injectable()
    ], TextRenderer);
    return TextRenderer;
}());
exports.TextRenderer = TextRenderer;
